<?php

namespace Drupal\cms_content_sync;

use Drupal\cms_content_sync\SyncCore\ApplicationInterface;
use Drupal\cms_content_sync\SyncCore\Client;

/**
 * Class DrupalApplication.
 *
 * Implement the ApplicationInterface to provide basic information about this site to the Sync Core. Must be provided
 * to all client instances to communicate with the Sync Core.
 *
 * This is the central interface between Drupal and the Sync Core. Drupal will call ::getClient() and then use the
 * Sync Core library whereas the Sync Core library in return uses the interface functions provided here.
 */
class SyncCore implements ApplicationInterface {
  /**
   * @var string APPLICATION_ID Unique ID to identify the kind of application.
   */
  const APPLICATION_ID = 'drupal';

  /**
   * @var SyncCore
   */
  protected static $instance;

  /**
   * @return SyncCore
   */
  public static function get() {
    if (!empty(self::$instance)) {
      return self::$instance;
    }

    return self::$instance = new SyncCore();
  }

  /**
   * Return an instance of the SyncCore Client for the given URL. Cache clients.
   *
   * @param string $sync_core_url
   *   The base URL to the Sync Core.
   *
   * @return \Drupal\cms_content_sync\SyncCore\Client
   */
  public static function getClient($sync_core_url) {
    static $clients = NULL;
    if (!empty($clients[$sync_core_url])) {
      return $clients[$sync_core_url];
    }

    return $clients[$sync_core_url] = new Client(self::get(), $sync_core_url);
  }

  /**
   *
   */
  public function getApplicationId() {
    return self::APPLICATION_ID;
  }

  /**
   *
   */
  public function getApplicationVersion() {
    return '8.x';
  }

  /**
   *
   */
  public function getApplicationModuleVersion() {
    return \Drupal::service('extension.list.module')->getExtensionInfo('cms_content_sync')['version'];
  }

  /**
   *
   */
  public function getHttpClient() {
    return \Drupal::httpClient();
  }

}
