<?php

namespace Drupal\cms_content_sync\SyncCore;

/**
 * Interface ApplicationInterface.
 *
 * Every application using the Sync Core library must implement this interface and provide it to the client. It provides
 * all basic information about the application itself to the Sync Core.
 */
interface ApplicationInterface {

  /**
   * Provide a unique identifier for the application. The following IDs are supported right now:
   * - drupal.
   *
   * @return string
   */
  public function getApplicationId();

  /**
   * Get a version (major, optionally minor) of the application as a string. For Drupal this is "8.x" right now.
   *
   * @return string
   */
  public function getApplicationVersion();

  /**
   * Get a version (major + minor) of the application module as a string.
   * For Drupal this is the published module version, e.g. "1.32".
   *
   * @return string
   */
  public function getApplicationModuleVersion();

  /**
   * Provide an HTTP client to use for all requests against the Sync Core. This allows the application to apply any
   * custom settings for the client name, timeouts, other security measures to be applied.
   *
   * @return \GuzzleHttp\Client
   */
  public function getHttpClient();

}
