<?php

namespace Drupal\cms_content_sync\SyncCore\Query;

use Drupal\cms_content_sync\SyncCore\Client;
use Drupal\cms_content_sync\SyncCore\Query\Result\SimpleResult;

/**
 * Class PingQuery.
 *
 * Ask the Sync Core to ping this site to make sure it's accessible.
 *
 * @package Drupal\cms_content_sync\SyncCore
 */
class PingQuery extends SimpleQuery {
  /**
   * @var array
   */
  protected $body;

  const PATH = '/ping';

  /**
   * Query constructor.
   *
   * @param \Drupal\cms_content_sync\SyncCore\Client $client
   */
  public function __construct($client) {
    parent::__construct($client, self::PATH);
  }

  /**
   * @inheritdoc
   */
  public static function create($client) {
    return new PingQuery($client);
  }

  /**
   * @inheritdoc
   */
  public function getMethod() {
    return Client::METHOD_POST;
  }

  /**
   * @inheritdoc
   */
  public function getBody() {
    return $this->body;
  }

  /**
   * @param string $set
   *
   * @return $this
   */
  public function setSiteUrl($set) {
    $this->body['url'] = $set;
    return $this;
  }

  /**
   * @param string $set
   *
   * @return $this
   */
  public function setMethod($set) {
    $this->body['method'] = $set;
    return $this;
  }

  /**
   * @param string $set
   *
   * @return $this
   */
  public function setAuthentication($set) {
    $this->body['authentication'] = $set;
    return $this;
  }

  /**
   * Provide a Result object to get the actual entities from.
   *
   * @return \Drupal\cms_content_sync\SyncCore\Query\Result\SimpleResult
   *
   * @throws \Drupal\cms_content_sync\SyncCore\Exception\SyncCoreException
   */
  public function execute() {
    $result = new SimpleResult($this);

    $result->execute();

    return $result;
  }

}
