<?php

namespace Drupal\cms_content_sync\SyncCore\Query;

use Drupal\cms_content_sync\SyncCore\Client;

/**
 * Class Query
 * A query to execute against the Sync Core. Will return a Result object when
 * executed. This is just a simple helper class to simplify query creation in
 * an OOP fashion.
 *
 * @package Drupal\cms_content_sync\SyncCore
 */
abstract class Query {

  /**
   * @var array
   */
  protected $arguments;

  /**
   * @var \Drupal\cms_content_sync\SyncCore\Client
   */
  protected $client;

  /**
   * Query constructor.
   *
   * @param \Drupal\cms_content_sync\SyncCore\Client $client
   * @param array $arguments
   */
  public function __construct($client, $arguments = []) {
    $this->client = $client;
    $this->arguments = $arguments;
  }

  /**
   * @return \Drupal\cms_content_sync\SyncCore\Client
   */
  public function getClient() {
    return $this->client;
  }

  /**
   * Get the arguments stored.
   *
   * @return array
   */
  public function toArray() {
    return $this->arguments;
  }

  /**
   * Get the path to be appended after the Pool::backend_url for this Query.
   *
   * @return string
   */
  abstract public function getPath();

  /**
   * Get the HTTP method to use for the request.
   *
   * @return string
   */
  public function getMethod() {
    return Client::METHOD_GET;
  }

  /**
   * Get the request body to use.
   *
   * @return string|null
   */
  public function getBody() {
    return NULL;
  }

  /**
   * @return bool|string
   */
  public function returnBoolean() {
    return FALSE;
  }

  /**
   * Provide a Result object to get the actual entities from.
   *
   * @return \Drupal\cms_content_sync\SyncCore\Query\Result\Result
   */
  abstract public function execute();

}
