<?php

namespace Drupal\cms_content_sync;

/**
 *
 */
abstract class SyncCoreExport {
  /**
   * @var \Drupal\cms_content_sync\SyncCore\Client
   */
  protected $client;

  /**
   * @param \Drupal\cms_content_sync\SyncCore\Client $client
   */
  public function __construct($client) {
    $this->client = $client;
  }

  /**
   * @return \Drupal\cms_content_sync\SyncCore\Client
   */
  public function getClient() {
    return $this->client;
  }

  /**
   * Prepare the Sync Core push as a batch operation. Return a batch array
   * with single steps to be executed.
   *
   * @return array Steps
   */
  abstract public function prepareBatch();

  /**
   * Execute a single batch step (as returned as an item from
   * {@see self::prepareBatch()}.
   * {@see SyncCoreBatchCollection}.
   *
   * @param array $operation
   *   Proving 'type' and 'item'.
   *
   * @return bool
   *
   * @throws SyncCore\Exception\SyncCoreException
   */
  public function executeBatch($operation) {
    return $this->sendEntityRequest($operation['type'], $operation['item']);
  }

  /**
   *
   */
  abstract public function remove($removedOnly = TRUE);

  /**
   * Send a request to the Sync Core backend.
   * Requests will be passed to $this->client.
   *
   * @param string $type
   * @param array $item
   *
   * @return bool
   *
   * @throws SyncCore\Exception\SyncCoreException
   */
  protected function sendEntityRequest($type, $item) {
    return $this
      ->client
      ->getStorageById($type)
      ->createItem($item)
      ->execute()
      ->succeeded();
  }

}
